#include <cstdio>
#include <yaco/chrono.hpp>
#include <yaco/collection.hpp>
#include <yaco/data_handler.hpp>
#include <yaco/grid.hpp>

int main(int argc, char** argv)
{
    if (argc != 2) {
        printf("Usage: %s <filename>\n", argv[0]);
        return 1;
    }

    yaco::chrono::setCalendar(yaco::chrono::Calendar::gregorian);
    yaco::chrono::DateTime time("1979-01-02T00:00:00Z");

    auto nc   = yaco::DataHandler::createNetCDFWriter(yaco::config::Document::fromYaml(argv[1]).value());
    auto grid = yaco::Grid::createHealpix("healpix_1", 1, yaco::Grid::HealpixScheme::ring);

    {
        yaco::Collection collection("10m_wind_speed", 1, *grid);
        for (int i = 0; i < collection.size(); ++i) {
            auto& d = collection.at(i);
            for (int j = 0; j < d.size(); ++j) {
                d[j] = j;
            }
        }
        nc->handle(collection, time);
    }
    {
        yaco::Collection collection("ta", 2, *grid);
        for (int i = 0; i < collection.size(); ++i) {
            auto& d = collection.at(i);
            for (int j = 0; j < d.size(); ++j) {
                d[j] = j;
            }
        }
        nc->handle(collection, time);
    }

    return 0;
}
