#include <libfyaml.h>
#include <yaco/config.hpp>

using yaco::config::Document;
using yaco::config::Value;

static Value createValueFromYamlNode(struct fy_node* node)
{
    fy_node_type type = fy_node_get_type(node);
    switch (type) {
        case FYNT_SCALAR:
            return Value(std::string(fy_node_get_scalar0(node)));
        case FYNT_SEQUENCE: {
            Value::Array vec;
            void* iter = nullptr;
            for (fy_node* child = fy_node_sequence_iterate(node, &iter); child;
                 child          = fy_node_sequence_iterate(node, &iter)) {
                vec.push_back(createValueFromYamlNode(child));
            }
            return Value(vec);
        }
        case FYNT_MAPPING: {
            Value::Map map;
            void* iter = nullptr;
            for (struct fy_node_pair* pair = fy_node_mapping_iterate(node, &iter); pair;
                 pair                      = fy_node_mapping_iterate(node, &iter)) {
                struct fy_node* key           = fy_node_pair_key(pair);
                struct fy_node* val           = fy_node_pair_value(pair);
                map[fy_node_get_scalar0(key)] = createValueFromYamlNode(val);
            }
            return Value(map);
        }
        default:
            throw std::runtime_error("Invalid node type");
    }
}

Document Document::fromYaml(const std::string& yml)
{
    Document doc;
    struct fy_document* fyd = fy_document_build_from_file(nullptr, yml.c_str());
    if (!fyd) {
        throw std::runtime_error("Failed to parse YAML");
    }
    doc.value_ = createValueFromYamlNode(fy_document_root(fyd));
    return doc;
}
